/*
 * Registration.java
 * Created on Jun 10, 2003
 *
 * Copyright (c) 2003 Harish Krishnaswamy
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
 * SOFTWARE. 
 */
package net.cc.hk;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.html.BasePage;

/**
 * @author  Harish Krishnaswamy
 */
public abstract class Registration extends BasePage {
	public static final SimpleDateFormat DATE_FORMAT =
		new SimpleDateFormat("dd MMM yyyy");

	// Abstract methods that the framework will implement, don't bother!
	public abstract String getCity();

	public abstract String getCustAddress();

	public abstract Date getCustDOB();

	public abstract String getCustName();

	public abstract String getZip();

	public abstract void setCity(String string);

	public abstract void setCustAddress(String string);

	public abstract void setCustDOB(Date date);

	public abstract void setCustName(String string);

	public abstract void setZip(String zip);

	public abstract String getState();

	public abstract void setState(String string);

	public abstract boolean isValidationError();

	public abstract void setValidationError(boolean flag);

	/**
	 * The listener method that will be called when the registration page is 
	 * submitted.
	 * @param cycle
	 */
	public void register(IRequestCycle cycle) {
		String custName = this.getCustName();
		String custAdd = this.getCustAddress();

		this.setCustName(custName.toUpperCase());
		this.setCustAddress(custAdd.toUpperCase());
		this.setCity(this.getCity().toUpperCase());
		this.setState(this.getState().toUpperCase());

		// NOTE: Although this kind of validation works, there is a better way of 
		// performing validaions in Tapestry using the validation framework.

		if (custName == null
			|| custAdd == null
			|| this.getCustDOB() == null
			|| custName.trim().length() == 0
			|| custAdd.trim().length() == 0) {

			// if validation fails
			// Set the validation error flag in this page to be used later
			// while rendering this page
			this.setValidationError(true);

			// If you do not forward to any page, the framework will by default 
			// take you back to the Page you came from (Registration Page).

		} else {
			// Upon successful request processing, forward request to "success" page
			// "success" page is defined in the specification in a declarative fashion
			Registration verificationPage =
				(Registration) cycle.getPage("Verification");

			copyTo(verificationPage);

			cycle.activate("Verification");
		}
	}

	private void copyTo(Registration to) {
		to.setCustName(this.getCustName());
		to.setCustAddress(this.getCustAddress());
		to.setCustDOB(this.getCustDOB());
		to.setCity(this.getCity());
		to.setState(this.getState());
		to.setZip(this.getZip());
	}

	/**
	 * The listener method that will be called when the verification page is 
	 * confirmed.
	 * @param cycle
	 */
	public void reenterRegistration(IRequestCycle cycle) {
		// Upon confirmation, you may update DB or do whatever...
		Registration registrationPage =
			(Registration) cycle.getPage("Registration");

		copyTo(registrationPage);

		cycle.activate("Registration");
	}

	/**
	 * The listener method that will be called when the verification page is 
	 * confirmed.
	 * @param cycle
	 */
	public void confirmRegistration(IRequestCycle cycle) {
		// Upon confirmation, you may update DB or do whatever...

		cycle.activate("Confirmation");
	}
}
